import numpy
import pandas

from lmtanalysis import Measure

class DetectionFeatures():
    @staticmethod
    def total_distance(df, region=None):
        name = "Distance"
        magic_thres = 85.5
        dists = numpy.sqrt(numpy.square(df[["x", "y"]].diff()).sum(1))

        if region is None:
            value = dists[dists <= magic_thres].sum()
        else:
            value = dists[df[region] & (dists <= magic_thres)].sum()
            name += f"_{region}"

        return pandas.Series({name: value * Measure.scaleFactor})

    @staticmethod
    def total_time(df, region=None):
        name = "Time_spent"
        dists = numpy.sqrt(numpy.square(df[["sec"]].diff()).sum(1))

        if region is None:
            value = dists.sum()
        else:
            value = dists[df[region]].sum()
            name += f"_{region}"

        return pandas.Series({name: value })

    @staticmethod
    def speed_avg(df, region=None):
        name = "Speed_average"
        if region is not None:
            name += f"_{region}"
        return pandas.Series({name: DetectionFeatures.total_distance(df, region).values[0] /
                                        DetectionFeatures.total_time(df, region).values[0]})

class EventFeatures():
    @staticmethod
    def duration_stats(df):
        return pandas.Series({
                            "Duration_mean":   df["duration"].mean(),
                            "Duration_median": df["duration"].median(),
                            "Duration_std":    df["duration"].std(),
                            })


def computeDetectionFeatures(animal_pool, start="0min", end="60min", freq="5min"):
    """Computes common features of the mouse trajectories in the animal pool,
       for a given time interval range given by start, end and freq (temporal bin size).

       Use pandas convention to specifiy time parameters.
           * 3 hours  : "3H"
           * 5 minutes: "5min"

        Features computed:
            * time spent
            * distance travelled
            * average speed


    Args:
        animal_pool (AnimalPool): the animal pool
        start (str, optional): Start time. Defaults to "0min".
        end (str, optional)  : End time.   Defaults to "60min".
        freq (str, optional) : Time step.  Defaults to "5min".

    Returns:
        DataFrame: Multiindex Dataframe containing mouse meta information and
                   computed features:

    """
    detection_table = animal_pool.getDetectionTable()
    time_delta_rng = pandas.timedelta_range(start=start, end=end, freq=freq)

    grp = detection_table.groupby(["RFID", pandas.cut(detection_table.time, bins=time_delta_rng)])
    return pandas.concat([
                        grp.name.first(),
                        grp.genotype.first(),
                        grp.apply(DetectionFeatures.total_time),
                        grp.apply(DetectionFeatures.total_time, "in_arena_center"),

                        grp.apply(DetectionFeatures.total_distance),
                        grp.apply(DetectionFeatures.total_distance, "in_arena_center"),

                        grp.apply(DetectionFeatures.speed_avg),
                        grp.apply(DetectionFeatures.speed_avg, "in_arena_center")
                    ], axis=1)

def computeEventFeatures(animal_pool, start="0min", end="60min", freq="5min"):
    """Computes common features of the mice' events in the animal pool,
       for a given time interval range given by start, end and freq (temporal bin size).

       Use pandas convention to specifiy time parameters.
           * 3 hours  : "3H"
           * 5 minutes: "5min"

        Features computed:
            * number of events
            * duration of events
                * mean/median/std


    Args:
        animal_pool (AnimalPool): the animal pool
        start (str, optional): Start time. Defaults to "0min".
        end (str, optional)  : End time.   Defaults to "60min".
        freq (str, optional) : Time step.  Defaults to "5min".

    Returns:
        DataFrame: Multiindex Dataframe containing mouse meta information and
                   computed event features:

    """
    events_table = animal_pool.getAllEventsTable()
    time_delta_rng = pandas.timedelta_range(start=start, end=end, freq=freq)

    grp = events_table.groupby(["event_name", "RFID",  pandas.cut(events_table.time, bins=time_delta_rng)])
    res = pandas.concat([
                   grp.name.first(),
                   grp.genotype.first(),
                   grp.size(),
                   grp.apply(EventFeatures.duration_stats)
    ], axis=1)
    res = res.rename(columns={0: "Number_of_events"})
    return res


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *

from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.Chronometer import Chronometer

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Detection" )
    
    
def loadDetectionMap( connection, animal, start=None, end=None ):
    
        chrono = Chronometer("Build event detection: Load detection map")
        # print( "processing animal ID: {}".format( animal ))

        result = {}
                
        cursor = connection.cursor()
        query = "SELECT FRAMENUMBER FROM DETECTION WHERE ANIMALID={}".format( animal )

        if ( start != None ):
            query += " AND FRAMENUMBER>={}".format(start )
        if ( end != None ):
            query += " AND FRAMENUMBER<={}".format(end )
            
        # print( query )
        cursor.execute( query )
        
        rows = cursor.fetchall()
        cursor.close()    
        
        for row in rows:
            frameNumber = row[0]
            result[frameNumber] = True;
        
        # print ( " detections loaded in {} seconds.".format( chrono.getTimeInS( )) )
        
        return result


def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ): 
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    for animal in range( 1 , 5 ):
        
        eventName = "Detection"        
        # print ( eventName )
            
        detectionTimeLine = EventTimeLine( None, eventName , animal , None , None , None , loadEvent=False )
         
        result = loadDetectionMap( connection , animal, tmin, tmax )
                                       
        #animal = pool.animalDictionnary[animal]
        #animal.loadDetection()
                    
        detectionTimeLine.reBuildWithDictionnary( result );
        detectionTimeLine.endRebuildEventTimeLine(connection)
    
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Detection" , tmin=tmin, tmax=tmax )

       
    # print( "Rebuild event finished." )
        
    


'''
Created on 2. February 2021

@author: Elodie
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "SniffFamiliar" )
    deleteEventTimeLineInBase(connection, "SniffNew" )
    deleteEventTimeLineInBase(connection, "SniffFamiliarFar")
    deleteEventTimeLineInBase(connection, "SniffNewFar")


def reBuildEvent( connection, objectPosition, radiusObjects, objectTuple, side, tmin=None, tmax=None, pool = None, vibrissae=3 ):
    deleteEventTimeLineInBase(connection, "SniffFamiliar" )
    deleteEventTimeLineInBase(connection, "SniffNew")
    deleteEventTimeLineInBase(connection, "SniffFamiliarFar")
    deleteEventTimeLineInBase(connection, "SniffNewFar")
    deleteEventTimeLineInBase(connection, "UpFamiliar")
    deleteEventTimeLineInBase(connection, "UpNew")


    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
        pool.filterDetectionByInstantSpeed(0, 70)
    '''
    Event SniffFamiliar
    - the animal's nose is in a zone 3 cm around the familiar object
    
    Event SniffNew
    - the animal's nose is in a zone 3 cm around the new object
    
    Event upFamiliar
    - the animal is on the familiar object
    
    Event upNew
    - the animal is on the new object
    '''
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
        
        eventNameSniffFamiliar = "SniffFamiliar"
        # print ( "A is around the familiar object")
        # print ( eventNameSniffFamiliar )
        
        eventNameSniffNew = "SniffNew"
        # print ( "A is around new object")
        # print ( eventNameSniffNew )

        eventNameSniffFamiliarFar = "SniffFamiliarFar"
        # print("A is around the familiar object, at a distance")
        # print(eventNameSniffFamiliarFar)

        eventNameSniffNewFar = "SniffNewFar"
        # print("A is around new object, at a distance")
        # print(eventNameSniffNewFar)

        eventNameUpFamiliar = "UpFamiliar"
        # print("A is on the familiar object")
        # print(eventNameUpFamiliar)

        eventNameUpNew = "UpNew"
        # print("A is on new object")
        # print(eventNameUpNew)
                
        sniffFamiliarTimeLine = EventTimeLine( None, eventNameSniffFamiliar , animal , None , None , None , loadEvent=False )
        sniffNewTimeLine = EventTimeLine( None, eventNameSniffNew , animal , None , None , None , loadEvent=False )
        sniffFamiliarTimeLineFar = EventTimeLine(None, eventNameSniffFamiliarFar, animal, None, None, None, loadEvent=False)
        sniffNewTimeLineFar = EventTimeLine(None, eventNameSniffNewFar, animal, None, None, None, loadEvent=False)
        upFamiliarTimeLine = EventTimeLine(None, eventNameUpFamiliar, animal, None, None, None, loadEvent=False)
        upNewTimeLine = EventTimeLine(None, eventNameUpNew, animal, None, None, None, loadEvent=False)

        resultSniffFamiliar = {}
        resultSniffNew = {}
        resultSniffFamiliarFar = {}
        resultSniffNewFar = {}
        resultUpFamiliar = {}
        resultUpNew = {}
        
        animalA = pool.animalDictionnary[animal]
        setup = animalA.setup
        objectFamiliar = objectTuple[side]
        objectNew = objectTuple[side]
        #print ( animalA )
        dicA = animalA.detectionDictionnary

        sideCage = {0: 'left', 1: 'right'}
        if side == 0:
            sideFam = 1
        elif side == 1:
            sideFam = 0

        for t in dicA.keys():
            #print(t)
            distanceNoseFamiliar = animalA.getDistanceNoseToPoint(t=t, xPoint=objectPosition[setup][sideCage[sideFam]][0], yPoint=-objectPosition[setup][sideCage[sideFam]][1])
            distanceMassFamiliar = animalA.getDistanceToPoint(t=t, xPoint=objectPosition[setup][sideCage[sideFam]][0], yPoint=-objectPosition[setup][sideCage[sideFam]][1])
            #print('distance left: ', distanceNoseFamiliar)
            if distanceNoseFamiliar == None:
                print('no nose detected for frame ', t)

            else:
                if distanceNoseFamiliar <= radiusObjects[objectFamiliar] + vibrissae / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassFamiliar <= radiusObjects[objectFamiliar]:
                        resultUpFamiliar[t] = True
                    else:
                        resultSniffFamiliar[t] = True

                if distanceNoseFamiliar <= radiusObjects[objectFamiliar] + 2*vibrissae / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassFamiliar > radiusObjects[objectFamiliar]:
                        resultSniffFamiliarFar[t] = True

            distanceNoseNew = animalA.getDistanceNoseToPoint(t=t, xPoint=objectPosition[setup][sideCage[side]][0], yPoint=-objectPosition[setup][sideCage[side]][1])
            distanceMassNew = animalA.getDistanceToPoint(t=t, xPoint=objectPosition[setup][sideCage[side]][0], yPoint=-objectPosition[setup][sideCage[side]][1])
            #print('distance right: ', distanceNoseNew)
            if distanceNoseNew == None:
                print('no nose detected for frame ', t)

            else:
                if distanceNoseNew <= radiusObjects[objectNew] + vibrissae / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassNew <= radiusObjects[objectNew]:
                        resultUpNew[t] = True
                    else:
                        resultSniffNew[t] = True

                if distanceNoseNew <= radiusObjects[objectNew] + 2*vibrissae / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassNew > radiusObjects[objectNew]:
                        resultSniffNewFar[t] = True

        sniffFamiliarTimeLine.reBuildWithDictionnary( resultSniffFamiliar )
        sniffFamiliarTimeLine.endRebuildEventTimeLine(connection)

        sniffNewTimeLine.reBuildWithDictionnary(resultSniffNew)
        sniffNewTimeLine.endRebuildEventTimeLine(connection)

        sniffFamiliarTimeLineFar.reBuildWithDictionnary(resultSniffFamiliarFar)
        sniffFamiliarTimeLineFar.endRebuildEventTimeLine(connection)

        sniffNewTimeLineFar.reBuildWithDictionnary(resultSniffNewFar)
        sniffNewTimeLineFar.endRebuildEventTimeLine(connection)

        upFamiliarTimeLine.reBuildWithDictionnary(resultUpFamiliar)
        upFamiliarTimeLine.endRebuildEventTimeLine(connection)

        upNewTimeLine.reBuildWithDictionnary(resultUpNew)
        upNewTimeLine.endRebuildEventTimeLine(connection)

    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Sniff Familiar" , tmin=tmin, tmax=tmax )
    t.addLog( "Build Event Sniff New" , tmin=tmin, tmax=tmax )
    t.addLog("Build Event Sniff Familiar Far", tmin=tmin, tmax=tmax)
    t.addLog("Build Event Sniff New Far", tmin=tmin, tmax=tmax)
    t.addLog("Build Event Up Familiar", tmin=tmin, tmax=tmax)
    t.addLog("Build Event Up New", tmin=tmin, tmax=tmax)

    # print( "Rebuild event finished." )
        
        
        
        
        
        
        
    


import sys
sys.path.insert(1, "../")
import os
import sqlite3
from lmtanalysis.FileUtil import getFilesToProcess, getFolderToProcess

# Select the file for processing
files = getFilesToProcess()

def SplitDatabase():
    # # Select the file for processing
    # files = getFilesToProcess()

    # Ensure there are files to process
    if not files:
        print("You have to select a least")
    else:
        # Use the first file to extract the directory
        first_file = files[0]
        initial_folder = os.path.dirname(first_file)

        # Ask the user for the new folder name
        new_folder_name = input("Enter the name of the new folder: ")

        # Concatenate the new folder with the initial directory
        save_folder = os.path.join(initial_folder, new_folder_name)

        # Check if the folder exists, if not, create it
        if not os.path.exists(save_folder):
            os.makedirs(save_folder)

        # Select the folder where the files will be created
        target_folder = getFolderToProcess()

        # Ensure the target folder is selected
        if not target_folder:
            print("No target folder selected. End of script.")
        else:
            # Call the function to process the files
            processFiles(files, save_folder)

            print("Database splited.")

def add_missing_columns(cursor, table_name, required_columns):
    cursor.execute(f"PRAGMA table_info('{table_name}')")
    existing_columns = set(row[1] for row in cursor.fetchall())

    missing_columns = set(required_columns) - existing_columns
    for column in missing_columns:
        cursor.execute(f"ALTER TABLE {table_name} ADD COLUMN {column} TEXT;")

def processFiles(files, save_folder):
    for file_path in files:
        connection = sqlite3.connect(file_path)
        cursor = connection.cursor()

        file_name = os.path.splitext(os.path.basename(file_path))[0]

        # Check if 'FRAME' table exists
        cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='FRAME';")
        frame_table_exists = cursor.fetchone()

        if frame_table_exists:
            # If 'FRAME' table exists, check the number of columns
            cursor.execute("PRAGMA table_info('FRAME')")
            frame_columns = cursor.fetchall()

            if len(frame_columns) == 4:
                # Add missing columns to 'FRAME' table
                required_columns = ['TEMPERATURE', 'HUMIDITY', 'SOUND', 'LIGHTVISIBLE', 'LIGHTVISIBLEANDIR']  # Replace with your actual column names
                add_missing_columns(cursor, 'FRAME', required_columns)

        cursor.execute("SELECT STARTFRAME, ENDFRAME FROM EVENT WHERE NAME = 'night'")
        create_table_query = cursor.fetchall()
        # print('This script will divide the input .sqlite file into different sqlite for each "night" event from the original database')
        print(f'The (start, stop) frames from {file_path} are:')
        print(create_table_query)

        for i, event in enumerate(create_table_query):
            print('     The following night is currently processed:')
            print(f"     {event}")

            start_frame = event[0]
            end_frame = event[1]

            output_file = os.path.join(save_folder, f"{file_name}-night{i + 1}.sqlite")
            new_connection = sqlite3.connect(output_file)

            new_cursor = new_connection.cursor()

            cursor.execute("SELECT sql FROM sqlite_master WHERE name='FRAME'")
            create_table_query = cursor.fetchone()[0]
            new_cursor.execute(create_table_query)

            cursor.execute("SELECT sql FROM sqlite_master WHERE name='DETECTION'")
            create_table_query = cursor.fetchone()[0]
            new_cursor.execute(create_table_query)

            cursor.execute("SELECT sql FROM sqlite_master WHERE name='EVENT'")
            create_table_query = cursor.fetchone()[0]
            new_cursor.execute(create_table_query)

            cursor.execute("SELECT sql FROM sqlite_master WHERE name='ANIMAL'")
            create_table_query = cursor.fetchone()[0]
            new_cursor.execute(create_table_query)

            cursor.execute("SELECT * FROM FRAME WHERE FRAMENUMBER BETWEEN ? AND ?", (start_frame, end_frame))
            frame_rows = cursor.fetchall()

            cursor.execute("PRAGMA table_info('FRAME')")
            columns_info = cursor.fetchall()
            # print(columns_info)

            # stmt = "INSERT INTO FRAME VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"
            stmt = "INSERT INTO FRAME (FRAMENUMBER, TIMESTAMP, NUMPARTICLE, PAUSED, TEMPERATURE, HUMIDITY, SOUND, LIGHTVISIBLE, LIGHTVISIBLEANDIR) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"
            new_cursor.executemany(stmt, frame_rows)

            cursor.execute("SELECT * FROM DETECTION WHERE FRAMENUMBER BETWEEN ? AND ?", (start_frame, end_frame))
            detection_rows = cursor.fetchall()

            stmt1 = "INSERT INTO DETECTION VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
            new_cursor.executemany(stmt1, detection_rows)

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='detectionFastLoadXYIndex'")
            existing_index = new_cursor.fetchone()

            if existing_index is None:
                new_cursor.execute(
                    "CREATE INDEX 'detectionFastLoadXYIndex' ON DETECTION (ANIMALID, FRAMENUMBER ASC, MASS_X, MASS_Y)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='detectionIndex'")
            existing_index = new_cursor.fetchone()

            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'detectionIndex' ON 'DETECTION' ('ID' ASC, 'FRAMENUMBER' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='detetIdIndex'")
            existing_index = new_cursor.fetchone()

            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'detetIdIndex' ON 'DETECTION' ('ID' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='detframenumberIndex'")
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'detframenumberIndex' ON 'DETECTION' ('FRAMENUMBER' ASC)")

            cursor.execute("SELECT * FROM ANIMAL")
            animal_rows = cursor.fetchall()

            stmt3 = "INSERT INTO ANIMAL VALUES (?, ?, ?, ?)"
            new_cursor.execute("DELETE FROM ANIMAL")
            new_cursor.executemany(stmt3, animal_rows)

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='animalINDEX'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX animalINDEX ON ANIMAL (ID)")

            cursor.execute("SELECT * FROM EVENT WHERE ENDFRAME BETWEEN ? AND ?", (start_frame, end_frame))

            event_rows = cursor.fetchall()

            stmt4 = "INSERT INTO EVENT VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
            new_cursor.executemany(stmt4, event_rows)

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='eventEndFrameIndex'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'eventEndFrameIndex' ON 'EVENT' ('ENDFRAME' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='eventIndex'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'eventIndex' ON 'EVENT' ('ENDFRAME' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='eventStartFrameIndex'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'eventStartFrameIndex' ON 'EVENT' ('ENDFRAME' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='eventstartendIndex'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'eventstartendIndex' ON 'EVENT' ('ENDFRAME' ASC)")

            new_cursor.execute("SELECT name FROM sqlite_master WHERE type='index' AND name='indexeventidIndex'")
            existing_index = new_cursor.fetchone()
            if existing_index is None:
                new_cursor.execute("CREATE INDEX 'indexeventidIndex' ON 'EVENT' ('ENDFRAME' ASC)")

            new_connection.commit()
            new_cursor.close()
            new_connection.close()

        cursor.close()
        connection.close()

# if __name__ == '__main__':
# # Appel de la fonction pour exécuter le workflow complet
# processFilesWorkflow()


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Rear in centerWindow" )
    deleteEventTimeLineInBase(connection, "Rear at periphery" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None  ):
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    rear = {}
    for animal in range( 1 , 5 ):
        rear[animal] = EventTimeLineCached( connection, file, "Rear isolated", animal, minFrame=tmin, maxFrame=tmax )
    
    centerWindow = {}
    for animal in range( 1 , 5 ):
        centerWindow[animal] = EventTimeLineCached(connection, file, "Center Zone", animal, minFrame=tmin, maxFrame=tmax )
    
    periphery = {}
    for animal in range( 1 , 5 ):
        periphery[animal] = EventTimeLineCached(connection, file, "Periphery Zone", animal, minFrame=tmin, maxFrame=tmax )
    
    
    for animal in range( 1 , 5 ):
        
        eventName1 = "Rear in centerWindow"        
        # print ( eventName1 )
        
        eventName2 = "Rear at periphery"
        # print ( eventName2 )
        
        rearCenterTimeLine = EventTimeLine( None, eventName1 , animal , None , None , None , loadEvent=False )
        rearPeripheryTimeLine = EventTimeLine( None, eventName2 , animal , None , None , None , loadEvent=False )
                   
        resultCenter={}
        resultPeriphery={}
        dicRear = rear[ animal ].getDictionnary()
        dicCenter = centerWindow[ animal ].getDictionnary()
        dicPeriphery = periphery[ animal ].getDictionnary()
        
        for t in dicRear.keys():
            if ( t in dicCenter ):
                resultCenter[t] = True
                
            if (t in dicPeriphery ):
                resultPeriphery[t] = True
                            
        rearCenterTimeLine.reBuildWithDictionnary( resultCenter )               
        rearPeripheryTimeLine.reBuildWithDictionnary( resultPeriphery )            
        
        rearCenterTimeLine.endRebuildEventTimeLine(connection)
        rearPeripheryTimeLine.endRebuildEventTimeLine(connection)
                    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Rear in centerWindow" , tmin=tmin, tmax=tmax )
    t.addLog( "Build Event Rear at periphery" , tmin=tmin, tmax=tmax ) 
                    
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Social escape" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None ): 
    
    # print("STARTING SOCIAL ESCAPE")
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
    
    nbAnimal = pool.getNbAnimals()

    # loading all the escape of animals    
    getAwayDico = {}
    for animal in range( 1 , nbAnimal+1 ):
        for idAnimalB in range( 1 , nbAnimal+1 ):
            if ( animal == idAnimalB ):
                continue
            
            getAwayDico[animal, idAnimalB] = EventTimeLineCached( connection, file, "Get away", animal, idAnimalB, minFrame=tmin, maxFrame=tmax )

    #cache mean body len
    twoMeanBodyLen = {}
    for idAnimal in range( 1 , nbAnimal+1 ):
        
        meanBodyLength = pool.animalDictionnary[idAnimal].getMeanBodyLength()
        # init value
        twoMeanBodyLen[idAnimal] = None
        # set value
        if meanBodyLength != None:
            twoMeanBodyLen[idAnimal] = 2*meanBodyLength
        
    
    for animal in range( 1 , nbAnimal+1 ):
        
        for idAnimalB in range( 1 , nbAnimal+1 ):
            if( animal == idAnimalB ):
                continue
            
            eventName = "Social escape"        
            # print ( eventName )
            
            socEscTimeLine = EventTimeLine( None, eventName , animal , idAnimalB , None , None , loadEvent=False )
                           
            result={}
            
            dicA = getAwayDico[ animal , idAnimalB ].getDictionnary()
            
            twoMeanBodyLengthB = twoMeanBodyLen[ idAnimalB ]
            
            for t in dicA.keys():
                dist = pool.animalDictionnary[animal].getDistanceTo(t, pool.animalDictionnary[idAnimalB])
                
                if ( dist == None ):
                    continue
                    
                if ( dist >= 0 and dist <= twoMeanBodyLengthB ):
                    result[t]=True
            
            
            socEscTimeLine.reBuildWithDictionnary( result )
            
            socEscTimeLine.endRebuildEventTimeLine(connection)
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Social Escape" , tmin=tmin, tmax=tmax )

             
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *

from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached
import networkx as nx

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Nest3_" )


def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    '''
    Nest 3
    ''' 
    # print("[NEST 3] : Assume that there is no occlusion, does not work with anonymous animals")
    
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax , lightLoad=True )
        
    if ( len ( pool.getAnimalList() ) != 4 ):
        # print( "[NEST3 Cancelled] 4 animals are required to build nest3.")
        return
    
    contact = {}
        
    for idAnimalA in range( 1 , 5 ):
        for idAnimalB in range( 1 , 5 ):
            if idAnimalA != idAnimalB:    
                contact[idAnimalA, idAnimalB] = EventTimeLineCached( connection, file, "Contact", idAnimalA, idAnimalB, minFrame=tmin, maxFrame=tmax ).getDictionnary()

    stopDictionnary = {}
        
    for idAnimalA in range( 1 , 5 ):
        stopDictionnary[idAnimalA] = EventTimeLineCached( 
            connection, file, "Stop", idA=idAnimalA, minFrame=tmin, maxFrame=tmax ).getDictionnary()
    
    nest3TimeLine = {}
    
    for idAnimalA in range( 1 , 5 ):
        # the id will be the one excluded from nest.
        nest3TimeLine[idAnimalA] = EventTimeLine( None, "Nest3_" , idA = idAnimalA , loadEvent=False )
    
    pool.loadAnonymousDetection()
    
    animalList = pool.getAnimalList() 
    
    result = {}
    for idAnimalA in range( 1 , 5 ):
        result[idAnimalA] = {}
    
    for t in range( tmin, tmax+1 ):
                
        isNest = False
        
        nbAnimalAtT = 0
        animalDetectedList = []
        
        anonymousDetectionList = pool.getAnonymousDetection( t )
        
        for animal in animalList:
            if t in animal.detectionDictionnary:
                nbAnimalAtT+=1
                animalDetectedList.append( animal )
        
        #print( str(t) + " : " + str( nbAnimalAtT ) )
                    
    
        #print("TEST")
        graph = nx.Graph()
        # add nodes
        for animal in animalDetectedList:
            graph.add_node( animal )
        for animalA in animalDetectedList:
            for animalB in animalDetectedList:
                if animalA != animalB:
                    # add an edge
                    if t in contact[animalA.baseId,animalB.baseId]:
                        graph.add_edge( animalA, animalB )
        
        # check with anonymous detection. Check contact
        if anonymousDetectionList!= None:
            # manage anonymous
            # print( t , "manage anonymous")
            '''
            # load all masks
            for animal in animalDetectedList:
                animal.loadMask( t )
            '''
            
            for detectionA in anonymousDetectionList: # anonymous with anonymous
                for detectionB in anonymousDetectionList: # anonymous with anonymous
                    if detectionA != detectionB:
                        distance = detectionA.getDistanceTo( detectionB )
                        if distance != None:
                            if distance < DISTANCE_CONTACT_MASS_CENTER:
                                graph.add_edge( detectionA, detectionB )
                                # print("Adding edge with mask (det anonymous to det anonymous)")
                    
            for detection in anonymousDetectionList:
                for animal in animalDetectedList:
                    distance = detection.getDistanceTo(animal.getDetectionAt( t ) )
                    if distance != None:
                        if distance < DISTANCE_CONTACT_MASS_CENTER:
                            #if detection.getMask().isInContactWithMask( animal.getDetectionAt ( t ).getMask() ):
                            graph.add_edge( animal, detection )
                            # print("Adding edge with mask")
        
        # list of CC from the biggest to the smallest
        listCC = sorted(nx.connected_components( graph ), key=len, reverse=True)
        
        if ( len( listCC ) == 2 ): # we have 2 groups
            
            # check if animals in the biggest group are stopped.
            allStoppedInBiggestGroup = True
            for animal in list( listCC[0] ):
                if isinstance( animal , Animal ):
                    if not ( t in stopDictionnary[animal.baseId] ):
                        allStoppedInBiggestGroup = False
                        break
                
            if allStoppedInBiggestGroup:
                if ( len( listCC[1] ) == 1 ): # the 2nd group (and the smallest) has only one mouse
                    animal = list(listCC[1])[0]
                    if isinstance( animal , Animal ):                
                        result[ animal.baseId ][ t ] = True
                 
            
    for idAnimalA in range( 1 , 5 ):
            
        # the id will be the one excluded from nest.
        nest3TimeLine[idAnimalA].reBuildWithDictionnary( result[idAnimalA] )
        # remove very small events
        nest3TimeLine[idAnimalA].removeEventsBelowLength( 2 )
        # merge flashing events
        nest3TimeLine[idAnimalA].mergeCloseEvents( 3 )
        nest3TimeLine[idAnimalA].endRebuildEventTimeLine(connection)
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Nest3" , tmin=tmin, tmax=tmax )
          
    

    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
from time import *

class Chronometer:
    '''
    classdocs
    '''


    def __init__(self, name):
        '''
        Constructor
        '''
        self.t = time()
        self.name= name
        
    # def printTimeInS(self):
    #     print ( "[Chrono " , self.name , " ] " , self.getTimeInS() , " seconds")
    #
    # def printTimeInMS(self):
    #     print ( "[Chrono " , self.name , " ] " , self.getTimeInMS() , " milliseconds")
    
    def getTimeInS(self):
        return time()-self.t
        
    def getTimeInMS(self):
        return ( time()-self.t ) * 1000
        
        
        


'''
Created on 21 mars 2019

@author: nicolas
'''

import sqlite3
from time import *
import datetime
from lmtanalysis.Util import *

from lmtanalysis.Animal import *
import matplotlib.pyplot as plt
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.FileUtil import getFilesToProcess
    
from psutil import virtual_memory

from tkinter.filedialog import askopenfilename
from lmtanalysis.TaskLogger import TaskLogger
import sys

class FileProcessException(Exception):
    pass

class Night(object):
    '''
    class Night
    '''

    def __init__(self, startHour=None, endHour=None, startDate=None, endDate=None, cycle="normal"):
        '''
        Constructor
        '''
        self.startHour=startHour
        self.endHour=endHour
        self.startDate=startDate
        self.endDate=endDate
        self.cycle = cycle

        def __str__(self):        
            return "Night begin at :{startHour} and stop at :{endHour} -  startDate: {startDate} -> endDate: {endDate}"\
                .format( startHour=self.startHour, endHour=self.endHour, startDate=self.startDate, endDate=self.endDate )   


    def getStartHour(self):
        return self.startHour
    
    def getEndHour(self):
        return self.endHour
    
    def getStartDate(self):
        return self.startDate
    
    def getEndDate(self):
        return self.endDate
    
    def getCycle(self):
        return self.cycle
    
    def setStartHour(self, startHour):
        self.startHour=startHour
        
    def setEndHour(self, endHour):
        self.endHour=endHour
    
    def setStartDate(self, startDate):
        self.startDate=startDate
        
    def setEndDate(self, endDate):
        self.endDate=endDate
        
    def setCycle(self, cycle):
        '''normal or reverse'''
        self.cycle = cycle

    def setStartEndDate(self, startDate):
        self.startDate=startDate
        day = datetime.datetime.strftime(self.startDate, "%Y-%m-%d")
        day = datetime.datetime(int(day.split("-")[0]),  int(day.split("-")[1]), int(day.split("-")[2]))
        if(self.cycle == "reverse"):
            day = datetime.datetime.strftime(day, "%Y-%m-%d")
            self.endDate = datetime.datetime.strptime("%s %s" % (day, self.endHour), "%Y-%m-%d %H:%M:%S")
        else:
            day += datetime.timedelta(days=1)
            day = datetime.datetime.strftime(day, "%Y-%m-%d")
            self.endDate = datetime.datetime.strptime("%s %s" % (day, self.endHour), "%Y-%m-%d %H:%M:%S")
            

    def nextDay(self):
        currentStartDay = datetime.datetime.strftime(self.startDate, "%Y-%m-%d")
        currentStartDay = datetime.datetime(int(currentStartDay.split("-")[0]),  int(currentStartDay.split("-")[1]), int(currentStartDay.split("-")[2]))
        currentEndDay = datetime.datetime.strftime(self.endDate, "%Y-%m-%d")
        currentEndDay = datetime.datetime(int(currentEndDay.split("-")[0]),  int(currentEndDay.split("-")[1]), int(currentEndDay.split("-")[2]))
        currentStartDay += datetime.timedelta(days=1)
        currentEndDay += datetime.timedelta(days=1)
        currentStartDay = datetime.datetime.strftime(currentStartDay, "%Y-%m-%d")
        currentEndDay = datetime.datetime.strftime(currentEndDay, "%Y-%m-%d")
        self.startDate = datetime.datetime.strptime("%s %s" % (currentStartDay, self.startHour), "%Y-%m-%d %H:%M:%S")
        self.endDate = datetime.datetime.strptime("%s %s" % (currentEndDay, self.endHour), "%Y-%m-%d %H:%M:%S")



def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "night" ) 

   
def insertNightEvent( file ):
    '''
    This function create night event
    '''
    
    connection = sqlite3.connect( file )     
                
    # print( "--------------")
    # print( "Current file: ", file )
    # print("Flush")
    flush( connection )
    
    # print( "--------------")
    # print( "Loading existing events...")
    nightTimeLine = EventTimeLine( connection, "night" , None, None, None , None )        
    
    # print( "--------------")
    # print( "Event list:" )
    # for event in nightTimeLine.eventList:
    #     print( event )
    # print( "--------------")
    
    
    startNightInput = input( "Time of the beginning of the night (hh:mm:ss):" )
    try:
        startNight = datetime.time(int(startNightInput.split(":")[0]),  int(startNightInput.split(":")[1]), int(startNightInput.split(":")[2]))
    except ValueError:
        raise ValueError("Incorrect time format, should be hh:mm:ss")

    endNightInput = input( "Time of the end of the night (hh:mm:ss):" )
    try:
        endNight = datetime.time(int(endNightInput.split(":")[0]),  int(endNightInput.split(":")[1]), int(endNightInput.split(":")[2]))
    except ValueError:
        raise ValueError("Incorrect time format, should be hh:mm:ss")
    
    """
    Two cases: 
    - end night hour < start night hour means end night hour is the day after
    - end night hour > start night hour means the night is during the day: reverse cycle
    """
    if (endNight < startNight):
        cycle = "normal"
    else:
        cycle = "reverse"
        
    currentNight = Night(startHour=startNight, endHour=endNight, cycle=cycle)
    
    '''Beginning and end of the experiment'''
    startExperimentDate = getStartInDatetime(file)
    endExperimentDate = getEndInDatetime(file)
    
    currentDay = datetime.datetime.strftime(startExperimentDate, "%Y-%m-%d")
    currentDay = datetime.datetime(int(currentDay.split("-")[0]),  int(currentDay.split("-")[1]), int(currentDay.split("-")[2]))
    previousDay = currentDay - datetime.timedelta(days=1)
    previousDay = datetime.datetime.strftime(previousDay, "%Y-%m-%d")
    currentStartNightDate = datetime.datetime.strptime("%s %s" % (previousDay, startNight), "%Y-%m-%d %H:%M:%S")
    
    lastFrame = getNumberOfFrames(file)
    
    currentNight.setStartEndDate(currentStartNightDate)
    
    while (True):
        if(currentNight.startDate > endExperimentDate):
            break
        
        tmpStartFrame = recoverFrame(file, str(currentNight.startDate))
        tmpEndFrame = recoverFrame(file, str(currentNight.endDate))
        
        if ((tmpStartFrame == 0) & (tmpEndFrame == 0)):
            if((currentNight.startDate < startExperimentDate) & (currentNight.endDate > endExperimentDate)):
                tmpStartFrame = 1
                tmpEndFrame = lastFrame
                nightTimeLine.addEvent( Event( tmpStartFrame, tmpEndFrame) )
                nightTimeLine.endRebuildEventTimeLine(connection)
            else:
                '''night outside the experiment'''
                pass
        else:
            if (tmpStartFrame == 0):
                tmpStartFrame = 1
            
            if (tmpEndFrame == 0):
                tmpEndFrame = lastFrame
                          
            nightTimeLine.addEvent( Event( tmpStartFrame, tmpEndFrame) )
            nightTimeLine.endRebuildEventTimeLine(connection)
        
        '''next day'''
        currentNight.nextDay()
  

if __name__ == '__main__':
    
    print("Code launched.")
    
    files = getFilesToProcess()

    chronoFullBatch = Chronometer("Full batch" )    
        
    if ( files != None ):
    
        for file in files:
            try:
                print ( "Processing file" , file )
                insertNightEvent( file )

            except FileProcessException:
                print ( "STOP PROCESSING FILE " + file , file=sys.stderr  )
              
    print( "*** ALL JOBS DONE ***")
        
  


'''
Created on 6 sept. 2017

@author: Fab
'''

import sys
import math
import time
import sqlite3
import datetime
import contextlib
from random import randrange, random
import string





class DummyFile(object):
    def write(self, x): pass

@contextlib.contextmanager
def mute_prints():
    # https://stackoverflow.com/questions/2828953/silence-the-stdout-of-a-function-in-python-without-trashing-sys-stdout-and-resto
    save_stdout = sys.stdout
    sys.stdout = DummyFile()
    yield
    sys.stdout = save_stdout

def getAllEvents(connection=None, file=None, ):
    """Returns names of all events present in database

    Args:
        file (str, optional): The path of the sqlite database. Defaults to None.
        connection (sqlite conn, optional): the sqlite connection. Defaults to None.

    Raises:
        ValueError: if , none of args have been supplied

    Returns:
        list(str): names of all events present in database
    """

    if not (file or connection):
        raise ValueError("Enter file or connection")
    elif file:
        connection = sqlite3.connect(file)

    print( "Loading event names:", end="")
    c = connection.cursor().execute( "SELECT name FROM event GROUP BY name" )
    all_rows = c.fetchall()
    print(f"{len(all_rows)} Done.")

    return [row[0] for row in all_rows]

def level( data ):
    ''' similar to level in R '''
    dico = {}
    for entry in data:
        dico[entry]=True
    return sorted( dico.keys( ) )


def pixelToCm( nbPixel ):
    return nbPixel * 10 / 57

def getFrameInput( text ):

    entryOk = False
    t = 0
    while not entryOk:
        try:
            t = 0
            r = input( text + " : ")

            if ( len( r ) == 0 ):
                return None
            answers  = r.split( sep=" ")

            for answer in answers:
                if answer.isdigit():
                    t+= float( answer )

                if answer.endswith("d"):
                    t+= float( answer[:-1] ) * 30 * 60 * 60 * 24

                if answer.endswith("h"):
                    t+= float( answer[:-1] ) * 30 * 60 * 60

                if answer.endswith("m"):
                    t+= float( answer[:-1] ) * 30 * 60

                if answer.endswith("s"):
                    t+= float( answer[:-1] ) * 30

                if answer.endswith("f"):
                    t+= float( answer[:-1] )
        except:
            print("Error in entry.")
            continue
        entryOk = True

    t = int(t)
    print("Entry (in frame) : " + str( t ) )
    return t


def getMinTMaxTAndFileNameInput():

    print ("Enter time information in frame. You can also set in days, hour, minutes")
    print ("valid entries: 100, 1d, 1.5d, 23.5h, 1d 2h 3m 4s 5f")


    tmin = getFrameInput("Starting t")
    tmax = getFrameInput("Ending t")

    text_file_name = input("Enter file name to save data (.txt will be added) : ")
    text_file_name = text_file_name+".txt"
    text_file = open ( text_file_name, "w")

    return tmin,tmax,text_file


def getMinTMaxTInput():

    print ("Enter time information in frame. You can also set in days, hour, minutes")
    print ("valid entries: 100, 1d, 1.5d, 23.5h, 1d 2h 3m 4s 5f")

    tmin = getFrameInput("Starting t")
    tmax = getFrameInput("Ending t")

    return tmin,tmax


def getFileNameInput():

    text_file_name = input("Please enter the 'File name' to be saved: ")
    text_file_name = text_file_name+".txt"
    text_file = open ( text_file_name, "w")

    return text_file

def convert_to_d_h_m_s( frames ):
    """Return the tuple of days, hours, minutes and seconds."""
    #seconds = frames / 30
    seconds, f = divmod( frames, 30)
    minutes, seconds = divmod(seconds, 60)
    hours, minutes = divmod(minutes, 60)
    days, hours = divmod(hours, 24)

    return days, hours, minutes, seconds, f

def d_h_m_s_toText( t ):
    """Return the d h m s f as text"""
    return "{} days {} hours {} minutes {} seconds {} frames".format( t[0],t[1],t[2],t[3],t[4] )

def getDistanceBetweenPointInPx( x1 , y1, x2, y2 ):
    ''' return the distance between two points in pixel '''
    distance = math.hypot( x1 - x2, y1 - y2 )
    return distance

def getNumberOfFrames(file):
    """Return the number of frame for a given experiment"""
    connection = sqlite3.connect( file )
    c = connection.cursor()
    query = "SELECT MAX(FRAMENUMBER) FROM FRAME";
    c.execute( query )
    numberOfFrames = c.fetchall()
    connection.close()
    return int(numberOfFrames[0][0])

def getStartInDatetime(file):
    """Return the start of a given experiment in a datetime format"""
    connection = sqlite3.connect( file )
    c = connection.cursor()
    query = "SELECT MIN(TIMESTAMP) FROM FRAME";
    c.execute( query )
    rows = c.fetchall()
    for row in rows:
        start = datetime.datetime.fromtimestamp(row[0]/1000)

    connection.close()
    return start

def getEndInDatetime(file):
    """Return the end of a given experiment in a datetime format"""
    connection = sqlite3.connect( file )
    c = connection.cursor()
    query = "SELECT MAX(TIMESTAMP) FROM FRAME";
    c.execute( query )
    rows = c.fetchall()
    for row in rows:
        end = datetime.datetime.fromtimestamp(row[0]/1000)

    connection.close()
    return end

def getDatetimeFromFrame(connection, frame):
    c = connection.cursor()
    query = "SELECT TIMESTAMP FROM FRAME WHERE FRAMENUMBER = {}".format(frame);

    c.execute( query )
    rows = c.fetchall()

    if (len(rows) <= 0):
        # print ("The entered framenumber is out of range")
        return None

    else:
        for row in rows:
            targetDate = datetime.datetime.fromtimestamp(row[0]/1000)

    return targetDate

def recoverFrame(file, MyDatetime):
    """
    Return the clothest FRAMENUMBER from a given datetime
    The datetime must have this format: dd-mm-YYYY hh:mm:ss
    """
    connection = sqlite3.connect( file )
    c = connection.cursor()

    # get timedate of 1st and last frame

    query = "SELECT FRAMENUMBER, TIMESTAMP FROM FRAME WHERE FRAMENUMBER=1";
    c.execute( query )
    all_rows = c.fetchall()
    startTS = int ( int ( all_rows[0][1] ) / 1000 )
    startDate = datetime.datetime.fromtimestamp( startTS ).strftime('%d-%m-%Y %H:%M:%S')

    query = "SELECT max(FRAMENUMBER) FROM FRAME";
    c.execute( query )
    all_rows = c.fetchall()
    maxFrame = all_rows[0][0]

    query = "SELECT FRAMENUMBER, TIMESTAMP FROM FRAME WHERE FRAMENUMBER={}".format( maxFrame );
    c.execute( query )
    all_rows = c.fetchall()
    endTS = int ( int ( all_rows[0][1] ) / 1000 )
    endDate = datetime.datetime.fromtimestamp( endTS ).strftime('%d-%m-%Y %H:%M:%S')

    #print ( file )
    #print ( "Start date of record : " + startDate )
    #print ( "End date of record : " + endDate )

    #print(datetime.utcfromtimestamp(startTS).strftime('%Y/%m/%d %H:%M:%S'))
    # print(MyDatetime)

    timeStamp = time.mktime(datetime.datetime.strptime(MyDatetime, "%Y-%m-%d %H:%M:%S").timetuple()) * 1000

    # print( "TimeStamp * 1000 is : " + str( timeStamp ) )
    #
    # print( "Searching closest frame in database....")

    query = "SELECT FRAMENUMBER, TIMESTAMP FROM FRAME WHERE TIMESTAMP>{} AND TIMESTAMP<{}".format( timeStamp - 10000 , timeStamp + 10000 );

    c.execute( query )
    all_rows = c.fetchall()

    closestFrame = 0
    smallestDif = 100000000

    for row in all_rows:

        ts = int ( row[1] )
        dif = abs( ts - timeStamp )
        if ( dif < smallestDif ):
            smallestDif = dif
            closestFrame = int (row[0] )

    # print( "Closest Frame in selected database is: " + str( closestFrame ) )
    # print( "Distance to target: " + str( smallestDif ) + " milliseconds")
    return closestFrame


def addJitter(x, jit):
    newX = []
    for item in x:
        addedJitter = (random() * 2 - 1) * jit
        newX.append(item + addedJitter)

    return newX


def getStarsFromPvalues(pvalue=None, U=None, numberOfTests=1):
    stars = "NA"

    s1 = 0.05 / numberOfTests
    s2 = 0.01 / numberOfTests
    s3 = 0.001 / numberOfTests
    '''if pvalue == "NA":
        stars = "NA"'''

    if (pvalue == 0) & (U == 0):
        stars = "NA"

    elif (pvalue == 0) & (U != 0):
        stars = "***'"

    else:
        if pvalue < s3:
            stars = "***"
        if pvalue >= s3 and pvalue < s2:
            stars = "**"
        if pvalue >= s2 and pvalue < s1:
            stars = "*"
        if pvalue >= s1:
            stars = "ns"

    return stars

def getStartTestPhase(pool):
    cursor = pool.conn.cursor()
    query = "SELECT FRAMENUMBER, PAUSED FROM FRAME"
    try:
        cursor.execute(query)
    except:
        print("can't access data for PAUSED")

    rows = cursor.fetchall()
    cursor.close()

    frameNumberList = []
    pauseValueList = []
    for row in rows:
        pauseValue = row[1]
        if pauseValue == 1:
            frameNumberList.append(row[0])
    sortedFrameList = sorted(frameNumberList)

    lastPausedFrame = sortedFrameList[-1]
    startFrameTestPhase = lastPausedFrame + 1
    return startFrameTestPhase

def getColorGeno(geno):
    if geno=="WT-WT":
        return 'steelblue'
    if geno=="KO-KO":
        return 'darkorange'
    if geno=="Del/+-Del/+":
        return 'darkorange'
    if geno == 'WT':
        return 'steelblue'
    if geno == 'Del/+':
        return 'darkorange'

def getLetterList():
    letterList = list(string.ascii_uppercase)
    return letterList




'''

@author: Fab

In case of a nest, for instance, 2 animals can be seen as one detection. Which is wrong.
in that case only one animal is observed and this should not be considered in other labeling.

The purpose of this code is to remove those faulty situation by switching the identity of animals involved in such situation to anonymous.

This script should not be ran if there is occlusion in the scene. This script assumes all animals could be watched all the time.

WARNING: 
This script alters the lmtanalysis:
After running this script detection at t without all identity recognized will be all switched to anonymous !

'''
import sqlite3
from time import *

from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.FileUtil import getFilesToProcess

def loadDetectionMap(connection, animal, start=None, end=None):
    
        chrono = Chronometer("Correct detection integrity: Load detection map")
        # print("processing animal ID: {}".format(animal))

        result = {}
        
        cursor = connection.cursor()
        query = "SELECT FRAMENUMBER FROM DETECTION WHERE ANIMALID={}".format(animal)

        if (start != None):
            query += " AND FRAMENUMBER>={}".format(start)
        if (end != None):
            query += " AND FRAMENUMBER<={}".format(end)
            
        # print(query)
        cursor.execute(query)
        
        rows = cursor.fetchall()
        cursor.close()    
        
        for row in rows:
            frameNumber = row[0]
            result[frameNumber] = True;
        
        # print(" detections loaded in {} seconds.".format(chrono.getTimeI()))
        
        return result


def correct(connection, tmin=None, tmax=None):
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )

    cursor = connection.cursor()

    if tmin is None:
        query = "SELECT MIN(FRAMENUMBER) FROM FRAME"
        cursor.execute(query)
        minFrames = cursor.fetchall()
        for minFrame in minFrames:
            tmin = minFrame[0]

    if tmax is None:
        query = "SELECT MAX(FRAMENUMBER) FROM FRAME"
        cursor.execute(query)
        maxFrames = cursor.fetchall()
        for maxFrame in maxFrames:
            tmax = maxFrame[0]

    #Select the MAX ID of DETECTION
    query = "SELECT MAX(ID) FROM DETECTION"
    cursor.execute(query)
    maxIDtemp = cursor.fetchall()
    maxID = maxIDtemp[0][0]
    # print(maxID)

    '''
    get the number of expected animals
    if there is not all detections expected, switch all to anonymous
    '''
    
    validDetectionTimeLine = EventTimeLine(None, "IDs integrity ok", None, None, None, None, loadEvent=False)
    validDetectionTimeLineDictionnary = {}

    detectionTimeLine = {}

    for idAnimal in pool.getAnimalDictionnary():
        detectionTimeLine[idAnimal] = loadDetectionMap(connection, idAnimal, tmin, tmax)

    for t in range(tmin, tmax +1):
        
        valid = True
        for idAnimal in detectionTimeLine.keys():
            if not (t in detectionTimeLine[idAnimal]):
                valid = False
        if (valid):
            validDetectionTimeLineDictionnary[t] = True
    
    '''
    rebuild detection set
    '''
    
    #cursor = connection.cursor()

    countCorrection = 0

    for idAnimal in detectionTimeLine.keys():
        for t in range ( tmin , tmax +1 ):
            if ( t in detectionTimeLine[idAnimal] ):
                if not ( t in validDetectionTimeLineDictionnary ):
                    query = "UPDATE `DETECTION` SET `ANIMALID`=NULL WHERE `FRAMENUMBER`='{}';".format( t )
                    cursor.execute( query )
                    # print ( f"{countCorrection}: {query}" )
                    countCorrection += 1
    
    connection.commit()
    cursor.close()
    validDetectionTimeLine.reBuildWithDictionnary( validDetectionTimeLineDictionnary )
    validDetectionTimeLine.endRebuildEventTimeLine(connection )

    # print(f" =>  THERE WERE {countCorrection} CORRECTIONS IN THE DATABASE")
    # print(f" This represents approximately {countCorrection/maxID*100}% of Alteration of the database")

    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger(connection)
    t.addLog("Correct detection integrity", tmin=tmin, tmax=tmax)

       
    # print( "Rebuild event finished." )


if __name__ == '__main__':
    files = getFilesToProcess()

    for file in files:
        # print("Processing file", file)
        connection = sqlite3.connect(file)  # connect to database
        animalPool = AnimalPool()  # create an animalPool, which basically contains your animals
        animalPool.loadAnimals(connection)  # load infos about the animals

        correct(connection)

        connection.close()

    print("******* ALL JOBS DONE !!! *******")




'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Group 4 make" )
    deleteEventTimeLineInBase(connection, "Group 4 break" )


def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    

    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    
    ''' load contact matrix '''
    contact = {}
    group4In = {}
    group4Out = {}
    
    for animal in range( 1 , 5 ):
        contact[animal] = EventTimeLineCached( connection, file, "Contact", animal, minFrame=tmin, maxFrame=tmax )
        group4In[animal] = EventTimeLine( connection, "Group 4 make", animal, loadEvent=False )
        group4Out[animal] = EventTimeLine( connection, "Group 4 break", animal, loadEvent=False )
        

    ''' process group '''
    
    for animal in range( 1 , 5 ):
        for idAnimalB in range( 1 , 5 ):
            for idAnimalC in range( 1 , 5 ):
                for idAnimalD in range( 1 , 5 ):

                    ''' check impossible combination (avoid 2 animals with same id) '''
                    
                    test = {}
                    test[animal] = True
                    test[idAnimalB] = True
                    test[idAnimalC] = True
                    test[idAnimalD] = True
                    if not len ( test.keys() ) == 4:
                        continue 
                    
                    ''' process group '''

                    group4 = EventTimeLineCached( connection, file, "Group4", animal, idAnimalB, idAnimalC, minFrame=tmin, maxFrame=tmax )
                    
                    eventList = group4.getEventList()
                    
                    for event in eventList:
                        
                        t = event.startFrame-1
    
                        if ( not contact[animal].hasEvent( t ) ):                        
                            group4In[animal].addPunctualEvent( t )
    
                        if ( not contact[idAnimalB].hasEvent( t ) ):                        
                            group4In[idAnimalB].addPunctualEvent( t )
    
                        if ( not contact[idAnimalC].hasEvent( t ) ):                        
                            group4In[idAnimalC].addPunctualEvent( t )

                        if ( not contact[idAnimalD].hasEvent( t ) ):                        
                            group4In[idAnimalD].addPunctualEvent( t )
                            
                        t = event.endFrame+1
    
                        if ( not contact[animal].hasEvent( t ) ):                        
                            group4Out[animal].addPunctualEvent( t )
    
                        if ( not contact[idAnimalB].hasEvent( t ) ):                        
                            group4Out[idAnimalB].addPunctualEvent( t )
    
                        if ( not contact[idAnimalC].hasEvent( t ) ):                        
                            group4Out[idAnimalC].addPunctualEvent( t )

                        if ( not contact[idAnimalD].hasEvent( t ) ):                        
                            group4Out[idAnimalD].addPunctualEvent( t )

                
    ''' save all '''
    for idAnimal in range( 1 , 5 ):
        
        group4In[idAnimal].endRebuildEventTimeLine(connection)
        group4Out[idAnimal].endRebuildEventTimeLine(connection)

        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Group4 Make Break" , tmin=tmin, tmax=tmax )
                                                
    # print( "Rebuild event finished." )
        
    